@@ -2,6 +2,85 @@
 
 . "test/testlib.sh"
 
+begin_test "pre-push with good ref"
+(
+  set -e
+  reponame="pre-push-master-branch-required"
+  setup_remote_repo "$reponame"
+  clone_repo "$reponame" "$reponame"
+
+  git config "lfs.$(repo_endpoint "$GITSERVER" "$reponame").locksverify" false
+  git lfs track "*.dat"
+  echo "hi" > a.dat
+  git add .gitattributes a.dat
+  git commit -m "add a.dat"
+
+  refute_server_object "$reponame" 98ea6e4f216f2fb4b69fff9b3a44842c38686ca685f3f55dc48c5d3fb1107be4 "refs/heads/master"
+
+  # for some reason, using 'tee' and $PIPESTATUS does not work here
+  echo "refs/heads/master master refs/heads/master 0000000000000000000000000000000000000000" |
+    git lfs pre-push origin "$GITSERVER/$reponame" 2>&1 > push.log
+
+  assert_server_object "$reponame" 98ea6e4f216f2fb4b69fff9b3a44842c38686ca685f3f55dc48c5d3fb1107be4 "refs/heads/master"
+)
+end_test
+
+begin_test "pre-push with tracked ref"
+(
+  set -e
+  reponame="pre-push-tracked-branch-required"
+  setup_remote_repo "$reponame"
+  clone_repo "$reponame" "$reponame"
+
+  git config "lfs.$(repo_endpoint "$GITSERVER" "$reponame").locksverify" false
+  git lfs track "*.dat"
+  echo "hi" > a.dat
+  git add .gitattributes a.dat
+  git commit -m "add a.dat"
+
+  refute_server_object "$reponame" 98ea6e4f216f2fb4b69fff9b3a44842c38686ca685f3f55dc48c5d3fb1107be4 "refs/heads/tracked"
+
+  # for some reason, using 'tee' and $PIPESTATUS does not work here
+  echo "refs/heads/master master refs/heads/tracked 0000000000000000000000000000000000000000" |
+    git lfs pre-push origin master 2>&1 > push.log
+
+  assert_server_object "$reponame" 98ea6e4f216f2fb4b69fff9b3a44842c38686ca685f3f55dc48c5d3fb1107be4 "refs/heads/tracked"
+)
+end_test
+
+begin_test "pre-push with bad ref"
+(
+  set -e
+  reponame="pre-push-other-branch-required"
+  setup_remote_repo "$reponame"
+  clone_repo "$reponame" "$reponame"
+
+  git config "lfs.$(repo_endpoint "$GITSERVER" "$reponame").locksverify" false
+  git lfs track "*.dat"
+  echo "hi" > a.dat
+  git add .gitattributes a.dat
+  git commit -m "add a.dat"
+
+  refute_server_object "$reponame" 98ea6e4f216f2fb4b69fff9b3a44842c38686ca685f3f55dc48c5d3fb1107be4 "refs/heads/other"
+
+  # for some reason, using 'tee' and $PIPESTATUS does not work here
+  set +e
+  echo "refs/heads/master master refs/heads/master 0000000000000000000000000000000000000000" |
+    git lfs pre-push origin "$GITSERVER/$reponame" 2> push.log
+  pushcode=$?
+  set -e
+
+  if [ "0" -eq "$pushcode" ]; then
+    echo "expected command to fail"
+    exit 1
+  fi
+
+  grep 'Expected ref "refs/heads/other", got "refs/heads/master"' push.log
+
+  refute_server_object "$reponame" 98ea6e4f216f2fb4b69fff9b3a44842c38686ca685f3f55dc48c5d3fb1107be4 "refs/heads/other"
+)
+end_test
+
 begin_test "pre-push"
 (
   set -e
@@ -34,7 +113,7 @@ begin_test "pre-push"
   echo "refs/heads/master master refs/heads/master 0000000000000000000000000000000000000000" |
     git lfs pre-push origin "$GITSERVER/$reponame" 2>&1 |
     tee push.log
-  grep "(1 of 1 files)" push.log
+  grep "Uploading LFS objects: 100% (1/1), 3 B" push.log
 
   assert_server_object "$reponame" 98ea6e4f216f2fb4b69fff9b3a44842c38686ca685f3f55dc48c5d3fb1107be4
 )
@@ -104,7 +183,7 @@ begin_test "pre-push 307 redirects"
   echo "refs/heads/master master refs/heads/master 0000000000000000000000000000000000000000" |
     git lfs pre-push origin "$GITSERVER/redirect307/rel/$reponame.git/info/lfs" 2>&1 |
     tee push.log
-  grep "(0 of 0 files, 1 skipped)" push.log
+  grep "Uploading LFS objects: 100% (1/1), 3 B" push.log
 
   assert_server_object "$reponame" 98ea6e4f216f2fb4b69fff9b3a44842c38686ca685f3f55dc48c5d3fb1107be4
 
@@ -120,7 +199,7 @@ begin_test "pre-push 307 redirects"
   echo "refs/heads/master master refs/heads/master 0000000000000000000000000000000000000000" |
     git lfs pre-push origin "$GITSERVER/redirect307/abs/$reponame.git/info/lfs" 2>&1 |
     tee push.log
-  grep "(0 of 0 files, 1 skipped)" push.log
+  grep "Uploading LFS objects: 100% (1/1), 3 B" push.log
 )
 end_test
 
@@ -146,7 +225,7 @@ begin_test "pre-push with existing file"
   echo "refs/heads/master master refs/heads/master 0000000000000000000000000000000000000000" |
     git lfs pre-push origin "$GITSERVER/$reponame" 2>&1 |
     tee push.log
-  grep "(1 of 1 files)" push.log
+  grep "Uploading LFS objects: 100% (1/1), 4 B" push.log
 
   # now the file exists
   assert_server_object "$reponame" 7aa7a5359173d05b63cfd682e3c38487f3cb4f7f1d60659fe59fab1505977d4c
@@ -171,7 +250,7 @@ begin_test "pre-push with existing pointer"
   echo "refs/heads/master master refs/heads/master 0000000000000000000000000000000000000000" |
     git lfs pre-push origin "$GITSERVER/$reponame" 2>&1 |
     tee push.log
-  grep "(1 of 1 files)" push.log
+  grep "Uploading LFS objects: 100% (1/1), 4 B" push.log
 )
 end_test
 
@@ -222,7 +301,7 @@ begin_test "pre-push with missing pointer which is on server"
   echo "refs/heads/master master refs/heads/master 0000000000000000000000000000000000000000" |
     git lfs pre-push origin "$GITSERVER/$reponame" 2>&1 |
     tee push.log
-  grep "(1 of 1 files)" push.log
+  grep "Uploading LFS objects: 100% (1/1), 11 B" push.log
 
   # now the file exists
   assert_server_object "$reponame" "$contents_oid"
@@ -583,7 +662,7 @@ begin_test "pre-push with our lock"
   git commit -m "add unauthorized changes"
 
   GIT_CURL_VERBOSE=1 git push origin master 2>&1 | tee push.log
-  grep "Consider unlocking your own locked file(s)" push.log
+  grep "Consider unlocking your own locked files" push.log
   grep "* locked.dat" push.log
 
   assert_server_lock "$id"
@@ -631,7 +710,7 @@ begin_test "pre-push with their lock on lfs file"
       exit 1
     fi
 
-    grep "Unable to push 1 locked file(s)" push.log
+    grep "Unable to push locked files" push.log
     grep "* locked_theirs.dat - Git LFS Tests" push.log
 
     grep "ERROR: Cannot update locked files." push.log
@@ -687,7 +766,7 @@ begin_test "pre-push with their lock on non-lfs lockable file"
       exit 1
     fi
 
-    grep "Unable to push 2 locked file(s)" push.log
+    grep "Unable to push locked files" push.log
     grep "* large_locked_theirs.dat - Git LFS Tests" push.log
     grep "* tiny_locked_theirs.dat - Git LFS Tests" push.log
     grep "ERROR: Cannot update locked files." push.log
@@ -777,6 +856,96 @@ begin_test "pre-push disable locks verify on partial url"
 )
 end_test
 
+begin_test "pre-push locks verify 403 with good ref"
+(
+  set -e
+
+  reponame="lock-verify-master-branch-required"
+  setup_remote_repo "$reponame"
+  clone_repo "$reponame" "$reponame"
+
+  contents="example"
+  contents_oid="$(calc_oid "$contents")"
+  printf "$contents" > a.dat
+  git lfs track "*.dat"
+  git add .gitattributes a.dat
+  git commit --message "initial commit"
+
+  git config "lfs.$GITSERVER/$reponame.git.locksverify" true
+  git push origin master 2>&1 | tee push.log
+
+  assert_server_object "$reponame" "$contents_oid" "refs/heads/master"
+)
+end_test
+
+begin_test "pre-push locks verify 403 with good tracked ref"
+(
+  set -e
+
+  reponame="lock-verify-tracked-branch-required"
+  setup_remote_repo "$reponame"
+  clone_repo "$reponame" "$reponame"
+
+  contents="example"
+  contents_oid="$(calc_oid "$contents")"
+  printf "$contents" > a.dat
+  git lfs track "*.dat"
+  git add .gitattributes a.dat
+  git commit --message "initial commit"
+
+  git config push.default upstream
+  git config branch.master.merge refs/heads/tracked
+  git config "lfs.$GITSERVER/$reponame.git.locksverify" true
+  git push 2>&1 | tee push.log
+
+  assert_server_object "$reponame" "$contents_oid" "refs/heads/tracked"
+)
+end_test
+
+begin_test "pre-push locks verify 403 with explicit ref"
+(
+  set -e
+
+  reponame="lock-verify-explicit-branch-required"
+  setup_remote_repo "$reponame"
+  clone_repo "$reponame" "$reponame"
+
+  contents="example"
+  contents_oid="$(calc_oid "$contents")"
+  printf "$contents" > a.dat
+  git lfs track "*.dat"
+  git add .gitattributes a.dat
+  git commit --message "initial commit"
+
+  git config "lfs.$GITSERVER/$reponame.git.locksverify" true
+  git push origin master:explicit 2>&1 | tee push.log
+
+  assert_server_object "$reponame" "$contents_oid" "refs/heads/explicit"
+)
+end_test
+
+begin_test "pre-push locks verify 403 with bad ref"
+(
+  set -e
+
+  reponame="lock-verify-other-branch-required"
+  setup_remote_repo "$reponame"
+  clone_repo "$reponame" "$reponame"
+
+  contents="example"
+  contents_oid="$(calc_oid "$contents")"
+  printf "$contents" > a.dat
+  git lfs track "*.dat"
+  git add .gitattributes a.dat
+  git commit --message "initial commit"
+
+  git config "lfs.$GITSERVER/$reponame.git.locksverify" true
+  git push origin master 2>&1 | tee push.log
+  grep "failed to push some refs" push.log
+  refute_server_object "$reponame" "$contents_oid" "refs/heads/other"
+)
+end_test
+
 begin_test "pre-push locks verify 5xx with verification unset"
 (
   set -e
